package starter;

import exceptions.EmptyException;

/**
 * Stack ADT implemented using an array.
 *
 * @param <T> base type.
 */
public class ArrayStack<T> implements Stack<T> {
  private T[] data;
  private int capacity;
  private int numElements;

  /**
   * Construct an ArrayStack.
   */
  public ArrayStack() {
    capacity = 10;
    data = (T[]) new Object[capacity];
    numElements = 0;
  }

  @Override
  public boolean empty() {
    return numElements == 0;
  }

  @Override
  public T top() throws EmptyException {
    if (empty()) {
      throw new EmptyException();
    }
    return data[numElements - 1];
  }

  @Override
  public void pop() throws EmptyException {
    if (empty()) {
      throw new EmptyException();
    }
    numElements--;
  }

  @Override
  public void push(T value) {
    // Amortized Analysis
    //    O(1)
    //   O(1) + O(1) + .... + O(1) + O(n)
    //   < -----   n - 1 ------ >    n
    //  [ (n-1)O(1) + O(n) ] / n
    //  [ O(n) ] / n
    //  O(1)

    if (numElements == capacity) { // O(1)
      grow(); // O(n)
    }
    data[numElements] = value; // O(1)
    numElements++; // o(1)
  }

  // make a bigger data array
  private void grow() {
    // time O(n)
    // input      O(n)
    // aux space O(n)
    capacity++;
    T[] newArray = (T[]) new Object[capacity];
    for (int i = 0; i < numElements; i++) {
      newArray[i] = data[i];
    }
    data = newArray;
  }
}
